/*
 * Copyright (c) KylinSoft Co., Ltd. 2025.All rights reserved.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef MESSAGEHANDLER_H
#define MESSAGEHANDLER_H

#include <functional>
#include <memory>
#include <QMap>
#include <QObject>
#include <QStringList>

#include "log.h"

class MessageHandler : public QObject {

public:
    using ResultCallback = std::function<void(const QString &)>;

    explicit MessageHandler(QObject *parent = nullptr);

    void registerCommand(const QString &command,
                         std::function<QString(const QStringList &)> handler);

    void processMessage(const QString &rawMessage,
                        ResultCallback callback = nullptr);

private:
    QString handleLogLevel(const QStringList &args) const;

private:
    QMap<QString, std::function<QString(const QStringList &)>> commandHandlers;
};

#endif // MESSAGEHANDLER_H
