/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef SCREENMANAGER_H
#define SCREENMANAGER_H

#include <QRect>
#include <QSize>
#include <QScreen>
#include <QGuiApplication>

class ScreenManager {
public:
    ScreenManager() = default;
    ~ScreenManager() = default;

    static QRect getPrimaryScreenGeometry();
    static QSize getPrimaryScreenSize();

public:
    template<typename Func>
    static void screenRemoved(Func&& callback) {
        QObject::connect(
            qApp, &QGuiApplication::screenRemoved,
            std::forward<Func>(callback)
        );
    }

};

#endif // SCREENMANAGER_H
