/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef LOCALSETTINGS_H
#define LOCALSETTINGS_H

#include <QVariant>

class LocalSettings {
public:
    virtual ~LocalSettings() = default;

    virtual QVariant getValue(const QString &group, const QString &key,
                              const QVariant &defaultValue = QVariant()) = 0;
    virtual void setValue(const QString &group, const QString &key,
                          const QVariant &value) = 0;

protected:
    LocalSettings() = default;
};

#endif // LOCALSETTINGS_H
