/*
 * Copyright (c) KylinSoft Co., Ltd. 2025.All rights reserved.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef _KEYBOARDSERVICEPROXY_H_
#define _KEYBOARDSERVICEPROXY_H_

#include <QDBusAbstractInterface>
#include <QDBusInterface>
#include <QDBusPendingReply>
#include <QObject>

#include "log.h"

class KeyboardServiceProxy : public QDBusAbstractInterface {
    Q_OBJECT
public:
    explicit KeyboardServiceProxy(QObject *parent = nullptr);
    ~KeyboardServiceProxy() override = default;

Q_SIGNALS:
    void kbdStatusChanged();

public Q_SLOTS:
    QDBusPendingReply<int> GetKbdCount();
};

#endif
