/*
 * Copyright (c) KylinSoft Co., Ltd. 2023.All rights reserved.
*
* This program is free software: you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation, either version 3 of the License, or (at your option) any later
* version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
* details.
*
* You should have received a copy of the GNU General Public License along with
* this program. If not, see <https://www.gnu.org/licenses/>.
*/

const KEY_ESC = 9;
const KEY_1 = 10;
const KEY_2 = 11;
const KEY_3 = 12;
const KEY_4 = 13;
const KEY_5 = 14;
const KEY_6 = 15;
const KEY_7 = 16;
const KEY_8 = 17;
const KEY_9 = 18;
const KEY_0 = 19;
const KEY_MINUS = 20;
const KEY_EQUAL = 21;
const KEY_BACKSPACE = 22;
const KEY_TAB = 23;
const KEY_Q = 24;
const KEY_W = 25;
const KEY_E = 26;
const KEY_R = 27;
const KEY_T = 28;
const KEY_Y = 29;
const KEY_U = 30;
const KEY_I = 31;
const KEY_O = 32;
const KEY_P = 33;
const KEY_LEFTBRACE = 34;
const KEY_RIGHTBRACE = 35;
const KEY_ENTER = 36;
const KEY_LEFTCTRL = 37;
const KEY_A = 38;
const KEY_S = 39;
const KEY_D = 40;
const KEY_F = 41;
const KEY_G = 42;
const KEY_H = 43;
const KEY_J = 44;
const KEY_K = 45;
const KEY_L = 46;
const KEY_SEMICOLON = 47;
const KEY_APOSTROPHE = 48;
const KEY_GRAVE = 49;
const KEY_LEFTSHIFT = 50;
const KEY_BACKSLASH = 51;
const KEY_Z = 52;
const KEY_X = 53;
const KEY_C = 54;
const KEY_V = 55;
const KEY_B = 56;
const KEY_N = 57;
const KEY_M = 58;
const KEY_COMMA = 59;
const KEY_DOT = 60;
const KEY_SLASH = 61;
const KEY_RIGHTSHIFT = 62;
const KEY_LEFTALT = 64;
const KEY_SPACE = 65;
const KEY_CAPSLOCK = 66;
const KEY_F1 = 67;
const KEY_F2 = 68;
const KEY_F3 = 69;
const KEY_F4 = 70;
const KEY_F5 = 71;
const KEY_F6 = 72;
const KEY_F7 = 73;
const KEY_F8 = 74;
const KEY_F9 = 75;
const KEY_F10 = 76;
const KEY_F11 = 95;
const KEY_F12 = 96;
const KEY_RIGHTCTRL = 105;
const KEY_RIGHTALT = 108;
const KEY_UP = 111;
const KEY_LEFT = 113;
const KEY_RIGHT = 114;
const KEY_DOWN = 116;
const KEY_DELETE = 119;
const KEY_LEFTSUPER = 133;
const KEY_RIGHTSUPER = 134;
